@@ -539,7 +539,13 @@ namespace OpenBve {
 				CurrentScore.PassengerTimer = 5.0;
 			} else {
 				CurrentScore.PassengerTimer -= TimeElapsed;
-				if (CurrentScore.PassengerTimer < 0.0) CurrentScore.PassengerTimer = 0.0;
+				if (CurrentScore.PassengerTimer <= 0.0) {
+					if (TrainManager.PlayerTrain.Passengers.FallenOver) {
+						CurrentScore.PassengerTimer = 5.0;
+					} else {
+						CurrentScore.PassengerTimer = 0.0;
+					}
+				}
 			}
 		}
 
@@ -932,7 +938,7 @@ namespace OpenBve {
 					} else if (Stations[d].ArrivalTime >= 0.0) {
 						t = Stations[d].ArrivalTime;
 					}
-					if (train == TrainManager.PlayerTrain & Stations[d].IsTerminalStation) {
+					if (train == TrainManager.PlayerTrain & Stations[d].IsTerminalStation & Stations[d].DepartureTime < 0.0) {
 						settored = true;
 					} else if (t >= 0.0 & SecondsSinceMidnight < t - train.TimetableDelta) {
 						settored = true;
@@ -1033,23 +1039,77 @@ namespace OpenBve {
 			private double TimeLastProcessed;
 			private double CurrentInterval;
 			private bool BrakeMode;
+			private double CurrentSpeedFactor;
 			private double PersonalitySpeedFactor;
 			private int PowerNotchAtWhichWheelSlipIsObserved;
 			private double AtsChimeCancelPosition;
 			private int AtsChimeCancelSection;
+			private int LastStation;
 			// functions
 			internal SimpleHumanDriverAI(TrainManager.Train Train) {
 				this.TimeLastProcessed = 0.0;
 				this.CurrentInterval = 1.0;
 				this.BrakeMode = false;
-				this.PersonalitySpeedFactor = 0.9 + 0.1 * Generator.NextDouble();
+				this.PersonalitySpeedFactor = 0.8 + 0.2 * Generator.NextDouble();
+				this.CurrentSpeedFactor = this.PersonalitySpeedFactor;
 				this.PowerNotchAtWhichWheelSlipIsObserved = Train.Specs.MaximumPowerNotch + 1;
 				this.AtsChimeCancelPosition = Train.Cars[0].FrontAxle.Follower.TrackPosition + 600.0;
 				this.AtsChimeCancelSection = -1;
+				this.LastStation = -1;
 			}
 			internal override void Trigger(TrainManager.Train Train, double TimeElapsed) {
-				if (SecondsSinceMidnight - TimeLastProcessed >= CurrentInterval) {
+				if (TimeLastProcessed > SecondsSinceMidnight) {
+					/* 
+					 * When jumping backward in time, the time the AI has last processed
+					 * may be greater than the current time. Let's handle this situation.
+					 * */
 					TimeLastProcessed = SecondsSinceMidnight;
+				} else if (SecondsSinceMidnight - TimeLastProcessed >= CurrentInterval) {
+					TimeLastProcessed = SecondsSinceMidnight;
+					double spd = Train.Specs.CurrentAverageSpeed;
+					// personality
+					if (Train.Station >= 0 & Train.StationState == TrainManager.TrainStopState.Boarding) {
+						if (Train.Station != this.LastStation) {
+							this.LastStation = Train.Station;
+							double time;
+							if (Stations[Train.Station].ArrivalTime >= 0.0) {
+								time = Stations[Train.Station].ArrivalTime;
+							} else if (Stations[Train.Station].DepartureTime >= 0.0) {
+								time = Stations[Train.Station].DepartureTime - Stations[Train.Station].StopTime;
+							} else {
+								time = double.MinValue;
+							}
+							time -= Train.TimetableDelta;
+							if (time != double.MinValue) {
+								const double timeThreshold = 30.0;
+								const double adjustmentFactor = 0.03;
+								double diff = SecondsSinceMidnight - time;
+								if (diff < -timeThreshold) {
+									this.CurrentSpeedFactor -= adjustmentFactor * (-diff - timeThreshold);
+									if (this.CurrentSpeedFactor < 0.5) {
+										this.CurrentSpeedFactor = 0.5;
+									}
+								} else if (diff > timeThreshold) {
+									this.CurrentSpeedFactor += adjustmentFactor * (diff - timeThreshold);
+									if (this.CurrentSpeedFactor > 1.2) {
+										this.CurrentSpeedFactor = 1.2;
+									}
+								} else if (Math.Abs(diff) < 0.5 * timeThreshold) {
+									if (this.CurrentSpeedFactor < this.PersonalitySpeedFactor) {
+										this.CurrentSpeedFactor += 0.05;
+										if (this.CurrentSpeedFactor > this.PersonalitySpeedFactor) {
+											this.CurrentSpeedFactor = this.PersonalitySpeedFactor;
+										}
+									} else if (this.CurrentSpeedFactor > this.PersonalitySpeedFactor) {
+										this.CurrentSpeedFactor -= 0.05;
+										if (this.CurrentSpeedFactor < this.PersonalitySpeedFactor) {
+											this.CurrentSpeedFactor = this.PersonalitySpeedFactor;
+										}
+									}
+								}
+							}
+						}
+					}
 					// door states
 					bool doorsopen = false;
 					for (int i = 0; i < Train.Cars.Length; i++) {
@@ -1149,6 +1209,11 @@ namespace OpenBve {
 						// eb
 						TrainManager.AcknowledgeSafetySystem(Train, TrainManager.AcknowledgementType.Eb);
 					}
+					if (Train.Specs.Safety.Mode == TrainManager.SafetySystem.Atc & !Train.Specs.Safety.Atc.Transmitting) {
+						if (Train.Specs.Safety.Ats.AtsAvailable & spd < 0.277777777777778) {
+							Train.Specs.Safety.ModeChange = TrainManager.SafetySystem.None;
+						}
+					}
 					// do the ai
 					Train.Specs.CurrentConstSpeed = false;
 					TrainManager.ApplyHoldBrake(Train, false);
@@ -1170,6 +1235,7 @@ namespace OpenBve {
 							Train.Specs.Safety.ModeChange = TrainManager.SafetySystem.None;
 							CurrentInterval = 10.0;
 						} else {
+							CurrentInterval = 1.0;
 							TrainManager.ApplyNotch(Train, -1, true, 0, true);
 							if (Train.Cars[Train.DriverCar].Specs.BrakeType == TrainManager.CarBrakeType.AutomaticAirBrake) {
 								if (Train.Cars[Train.DriverCar].Specs.AirBrake.BrakeCylinderCurrentPressure < 0.3 * Train.Cars[Train.DriverCar].Specs.AirBrake.BrakeCylinderServiceMaximumPressure) {
@@ -1180,7 +1246,13 @@ namespace OpenBve {
 									TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Lap);
 								}
 							} else {
-								int b = (int)Math.Ceiling(0.5 * (double)Train.Specs.MaximumBrakeNotch);
+								int b;
+								if (Math.Abs(spd) < 0.02) {
+									b = (int)Math.Ceiling(0.5 * (double)Train.Specs.MaximumBrakeNotch);
+									CurrentInterval = 0.3;
+								} else {
+									b = Train.Specs.MaximumBrakeNotch;
+								}
 								if (Train.Specs.CurrentBrakeNotch.Driver < b) {
 									TrainManager.ApplyNotch(Train, 0, true, 1, true);
 								} else if (Train.Specs.CurrentBrakeNotch.Driver > b) {
@@ -1195,10 +1267,12 @@ namespace OpenBve {
 								}
 							} else if (Train.Station >= 0 & Train.StationState == TrainManager.TrainStopState.Boarding) {
 								// still boarding
-								if (Stations[Train.Station].SafetySystem == SafetySystem.Ats & Train.Specs.Safety.Mode != TrainManager.SafetySystem.AtsSn & Train.Specs.Safety.Mode != TrainManager.SafetySystem.AtsP) {
-									Train.Specs.Safety.ModeChange = TrainManager.SafetySystem.AtsSn;
-								} else if (Stations[Train.Station].SafetySystem == SafetySystem.Atc & Train.Specs.Safety.Mode != TrainManager.SafetySystem.Atc & Train.Specs.Safety.Atc.Available) {
-									Train.Specs.Safety.ModeChange = TrainManager.SafetySystem.Atc;
+								if (Train.Specs.Safety.Mode != TrainManager.SafetySystem.Plugin) {
+									if (Stations[Train.Station].SafetySystem == SafetySystem.Ats & Train.Specs.Safety.Ats.AtsAvailable & Train.Specs.Safety.Mode != TrainManager.SafetySystem.AtsSn & Train.Specs.Safety.Mode != TrainManager.SafetySystem.AtsP) {
+										Train.Specs.Safety.ModeChange = TrainManager.SafetySystem.AtsSn;
+									} else if (Stations[Train.Station].SafetySystem == SafetySystem.Atc & Train.Specs.Safety.Mode != TrainManager.SafetySystem.Atc & Train.Specs.Safety.Atc.Available) {
+										Train.Specs.Safety.ModeChange = TrainManager.SafetySystem.Atc;
+									}
 								}
 							} else {
 								// not at station - close doors
@@ -1206,7 +1280,6 @@ namespace OpenBve {
 									TrainManager.CloseTrainDoors(Train, true, true);
 								}
 							}
-							CurrentInterval = 1.0;
 						}
 					} else if (Train.Station >= 0 && stopIndex >= 0 && Train.StationDistanceToStopPoint < Stations[Train.Station].Stops[stopIndex].BackwardTolerance && (StopsAtStation(Train.Station, Train) & (Stations[Train.Station].OpenLeftDoors | Stations[Train.Station].OpenRightDoors) & Math.Abs(Train.Specs.CurrentAverageSpeed) < 0.25 & Train.StationState == TrainManager.TrainStopState.Pending)) {
 						// arrived at station - open doors
@@ -1214,9 +1287,16 @@ namespace OpenBve {
 							TrainManager.OpenTrainDoors(Train, Stations[Train.Station].OpenLeftDoors, Stations[Train.Station].OpenRightDoors);
 						}
 						CurrentInterval = 1.0;
+					} else if (Train.Station >= 0 && Stations[Train.Station].IsTerminalStation && Train == TrainManager.PlayerTrain && Train.StationDistanceToStopPoint < Stations[Train.Station].Stops[stopIndex].BackwardTolerance && -Train.StationDistanceToStopPoint < Stations[Train.Station].Stops[stopIndex].ForwardTolerance && Math.Abs(Train.Specs.CurrentAverageSpeed) < 0.25) {
+						// player's terminal station (not boarding any longer)
+						TrainManager.ApplyReverser(Train, 0, false);
+						TrainManager.ApplyNotch(Train, -1, true, 1, true);
+						TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Service);
+						TrainManager.ApplyEmergencyBrake(Train);
+						Train.Specs.Safety.ModeChange = TrainManager.SafetySystem.None;
+						CurrentInterval = 10.0;
 					} else {
 						// drive
-						TrainManager.UnapplyEmergencyBrake(Train);
 						TrainManager.ApplyReverser(Train, 1, false);
 						if (Train.Cars[Train.DriverCar].FrontAxle.CurrentWheelSlip | Train.Cars[Train.DriverCar].RearAxle.CurrentWheelSlip) {
 							// react to wheel slip
@@ -1229,7 +1309,6 @@ namespace OpenBve {
 							}
 						}
 						// initialize
-						double spd = Train.Specs.CurrentAverageSpeed;
 						double acc = Train.Specs.CurrentAverageAcceleration;
 						double lim = PrecedingTrainSpeedLimit * 1.2;
 						if (Train.Specs.Safety.Mode == TrainManager.SafetySystem.Atc) {
@@ -1250,10 +1329,10 @@ namespace OpenBve {
 							powerend = lim;
 							brakestart = lim;
 						} else {
-							lim *= this.PersonalitySpeedFactor;
+							lim *= this.CurrentSpeedFactor;
 							if (spd < 8.0) {
-								powerstart = 0.7 * lim;
-								powerend = 0.9 * lim;
+								powerstart = 0.75 * lim;
+								powerend = 0.95 * lim;
 							} else {
 								powerstart = lim - 2.5;
 								powerend = lim - 1.5;
@@ -1265,28 +1344,38 @@ namespace OpenBve {
 							}
 						}
 						double dec = 0.0;
-						double dectol;
+						double decelerationCruise;   /* power below this deceleration, cruise above */
+						double decelerationStart;    /* brake above this deceleration, cruise below */
+						double decelerationStep;     /* the deceleration step per brake notch */
 						double BrakeDeceleration = Train.Cars[Train.DriverCar].Specs.BrakeDecelerationAtServiceMaximumPressure;
-						if (Train.Cars[Train.DriverCar].Specs.BrakeType == TrainManager.CarBrakeType.AutomaticAirBrake | Train.Specs.MaximumBrakeNotch <= 0) {
-							dectol = 0.1 * BrakeDeceleration;
-						} else if (Train.Specs.MaximumBrakeNotch <= 2) {
-							dectol = 0.3 * BrakeDeceleration;
-						} else {
-							dectol = 0.9 * BrakeDeceleration / (double)Train.Specs.MaximumBrakeNotch;
-						}
-						if (spd > 0.0 & spd > brakestart) {
-							double r = 0.125 + (spd - powerend) / spd;
-							if (r > 0.0) {
-								if (r > 1.0) {
-									dec = BrakeDeceleration;
-								} else {
-									dec = r * BrakeDeceleration;
+						for (int i = 0; i < Train.Cars.Length; i++) {
+							if (Train.Cars[i].Specs.IsMotorCar) {
+								if (Train.Cars[Train.DriverCar].Specs.MotorDeceleration < BrakeDeceleration) {
+									BrakeDeceleration = Train.Cars[Train.DriverCar].Specs.MotorDeceleration;
 								}
+								break;
 							}
 						}
-						double cruisedec = 0.01 * BrakeDeceleration;
-						// lookahead
-						double lookahead = (Train.Station >= 0 ? 150.0 : 75.0) + (spd * spd) / BrakeDeceleration;
+						if (Train.Cars[Train.DriverCar].Specs.BrakeType == TrainManager.CarBrakeType.AutomaticAirBrake | Train.Specs.MaximumBrakeNotch <= 0) {
+							decelerationCruise = 0.3 * BrakeDeceleration;
+							decelerationStart = 0.5 * BrakeDeceleration;
+							decelerationStep = 0.1 * BrakeDeceleration;
+						} else if (Train.Specs.MaximumBrakeNotch <= 2) {
+							decelerationCruise = 0.2 * BrakeDeceleration;
+							decelerationStart = 0.4 * BrakeDeceleration;
+							decelerationStep = 0.5 * BrakeDeceleration;
+						} else {
+							decelerationCruise = 0.2 * BrakeDeceleration;
+							decelerationStart = 0.5 * BrakeDeceleration;
+							decelerationStep = BrakeDeceleration / (double)Train.Specs.MaximumBrakeNotch;
+						}
+						bool forceBrakeMode = false;
+						if (spd > 0.0 & spd > brakestart) {
+							dec = decelerationStep + 0.1 * (spd - brakestart);
+						}
+						bool reduceDecelerationCruiseAndStart = false;
+						// look ahead
+						double lookahead = (Train.Station >= 0 ? 150.0 : 50.0) + (spd * spd) / (2.0 * decelerationCruise);
 						double tp = Train.Cars[0].FrontAxle.Follower.TrackPosition - Train.Cars[0].FrontAxlePosition + 0.5 * Train.Cars[0].Length;
 						{
 							// events
@@ -1300,7 +1389,7 @@ namespace OpenBve {
 										TrackManager.LimitChangeEvent e = (TrackManager.LimitChangeEvent)TrackManager.CurrentTrack.Elements[i].Events[j];
 										if (e.NextSpeedLimit < spd) {
 											double dist = stp + e.TrackPositionDelta - tp;
-											double edec = (spd * spd - e.NextSpeedLimit * e.NextSpeedLimit) / (2.0 * dist);
+											double edec = (spd * spd - e.NextSpeedLimit * e.NextSpeedLimit * this.CurrentSpeedFactor) / (2.0 * dist);
 											if (edec > dec) dec = edec;
 										}
 									} else if (TrackManager.CurrentTrack.Elements[i].Events[j] is TrackManager.SectionChangeEvent) {
@@ -1309,7 +1398,7 @@ namespace OpenBve {
 											TrackManager.SectionChangeEvent e = (TrackManager.SectionChangeEvent)TrackManager.CurrentTrack.Elements[i].Events[j];
 											if (stp + e.TrackPositionDelta > tp) {
 												if (!Game.Sections[e.NextSectionIndex].Invisible & Game.Sections[e.NextSectionIndex].CurrentAspect >= 0) {
-													double elim = Game.Sections[e.NextSectionIndex].Aspects[Game.Sections[e.NextSectionIndex].CurrentAspect].Speed;
+													double elim = Game.Sections[e.NextSectionIndex].Aspects[Game.Sections[e.NextSectionIndex].CurrentAspect].Speed * this.CurrentSpeedFactor;
 													if (elim < spd | spd <= 0.0) {
 														double dist = stp + e.TrackPositionDelta - tp;
 														double edec;
@@ -1326,14 +1415,19 @@ namespace OpenBve {
 																redstopdist = 35.0;
 															} else if (Train.Specs.Safety.Mode == TrainManager.SafetySystem.AtsSn) {
 																redstopdist = 25.0;
+															} else if (Train.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) {
+																redstopdist = 35.0;
 															} else {
-																redstopdist = 15.0;
+																redstopdist = 10.0;
 															}
 															if (dist > redstopdist) {
 																edec = (spd * spd) / (2.0 * (dist - redstopdist));
 															} else {
 																edec = BrakeDeceleration;
 															}
+															if (dist < 100.0) {
+																reduceDecelerationCruiseAndStart = true;
+															}
 														} else {
 															if (dist >= 1.0) {
 																edec = (spd * spd - elim * elim) / (2.0 * dist);
@@ -1346,24 +1440,44 @@ namespace OpenBve {
 												}
 											}
 										}
-									} else if (TrackManager.CurrentTrack.Elements[i].Events[j] is TrackManager.StationEndEvent) {
-										// station stop
-										TrackManager.StationEndEvent e = (TrackManager.StationEndEvent)TrackManager.CurrentTrack.Elements[i].Events[j];
-										if (StopsAtStation(e.StationIndex, Train) & Train.StationState == TrainManager.TrainStopState.Pending) {
-											int s = GetStopIndex(e.StationIndex, Train.Cars.Length);
-											if (s >= 0) {
-												double dist = Stations[e.StationIndex].Stops[s].TrackPosition - tp;
-												if (dist > -Stations[e.StationIndex].Stops[s].ForwardTolerance) {
-													double Tback = Stations[e.StationIndex].Stops[s].BackwardTolerance;
-													if (dist >= Tback) {
-														double edec;
-														if (spd > 2.0) {
-															if (dist > 25.0) {
-																dist -= 7.5;
-															}
-															edec = spd * spd / (2.0 * dist);
-															if (edec > dec) dec = edec;
+									} else if (TrackManager.CurrentTrack.Elements[i].Events[j] is TrackManager.StationStartEvent) {
+										// station start
+										if (Train.Station == -1) {
+											TrackManager.StationStartEvent e = (TrackManager.StationStartEvent)TrackManager.CurrentTrack.Elements[i].Events[j];
+											if (StopsAtStation(e.StationIndex, Train) & Train.LastStation != e.StationIndex) {
+												int s = GetStopIndex(e.StationIndex, Train.Cars.Length);
+												if (s >= 0) {
+													double dist = Stations[e.StationIndex].Stops[s].TrackPosition - tp;
+													if (dist > -Stations[e.StationIndex].Stops[s].ForwardTolerance) {
+														if (dist < 25.0) {
+															reduceDecelerationCruiseAndStart = true;
+														} else {
+															dist -= 5.0;
 														}
+														double edec;
+														edec = spd * spd / (2.0 * dist);
+														if (edec > dec) dec = edec;
+													}
+												}
+											}
+										}
+									} else if (TrackManager.CurrentTrack.Elements[i].Events[j] is TrackManager.StationEndEvent) {
+										// station end
+										if (Train.Station == -1) {
+											TrackManager.StationEndEvent e = (TrackManager.StationEndEvent)TrackManager.CurrentTrack.Elements[i].Events[j];
+											if (StopsAtStation(e.StationIndex, Train) & Train.LastStation != e.StationIndex) {
+												int s = GetStopIndex(e.StationIndex, Train.Cars.Length);
+												if (s >= 0) {
+													double dist = Stations[e.StationIndex].Stops[s].TrackPosition - tp;
+													if (dist > -Stations[e.StationIndex].Stops[s].ForwardTolerance) {
+														if (dist < 25.0) {
+															reduceDecelerationCruiseAndStart = true;
+														} else {
+															dist -= 5.0;
+														}
+														double edec;
+														edec = spd * spd / (2.0 * dist);
+														if (edec > dec) dec = edec;
 													}
 												}
 											}
@@ -1385,24 +1499,55 @@ namespace OpenBve {
 								}
 							}
 						}
+						// trains ahead
+						for (int i = 0; i < TrainManager.Trains.Length; i++) {
+							if (TrainManager.Trains[i] != Train && TrainManager.Trains[i].State == TrainManager.TrainState.Available) {
+								double pos =
+									TrainManager.Trains[i].Cars[TrainManager.Trains[i].Cars.Length - 1].RearAxle.Follower.TrackPosition -
+									TrainManager.Trains[i].Cars[TrainManager.Trains[i].Cars.Length - 1].RearAxlePosition -
+									0.5 * TrainManager.Trains[i].Cars[TrainManager.Trains[i].Cars.Length - 1].Length;
+								double dist = pos - tp;
+								if (dist > -10.0 & dist < lookahead) {
+									const double minDistance = 8.0;
+									const double fastReactionDistance = 35.0;
+									double edec;
+									if (dist > minDistance) {
+										double shift = 0.75 * minDistance + 1.0 * spd;
+										edec = spd * spd / (2.0 * (dist - shift));
+									} else if (dist > 0.5 * minDistance) {
+										edec = BrakeDeceleration;
+									} else {
+										TrainManager.ApplyEmergencyBrake(Train);
+										TrainManager.ApplyNotch(Train, -1, true, 1, true);
+										this.CurrentInterval = 0.1;
+										return;
+									}
+									if (dist < fastReactionDistance) {
+										this.CurrentInterval = 0.1;
+									}
+									if (edec > dec) dec = edec;
+								}
+							}
+						}
+						TrainManager.UnapplyEmergencyBrake(Train);
 						// current station
-						if (Train.Station >= 0) {
-							if (StopsAtStation(Train.Station, Train) & Train.StationState == TrainManager.TrainStopState.Pending) {
+						if (Train.Station >= 0 & Train.StationState == TrainManager.TrainStopState.Pending) {
+							if (StopsAtStation(Train.Station, Train)) {
 								int s = GetStopIndex(Train.Station, Train.Cars.Length);
 								if (s >= 0) {
 									double dist = Stations[Train.Station].Stops[s].TrackPosition - tp;
-									double edec;
-									if (dist < 0.0) {
-										edec = BrakeDeceleration;
-									} else if (dist < Stations[Train.Station].Stops[s].BackwardTolerance | spd > 0.5) {
-										if (dist > 25.0) {
-											dist -= 7.5;
+									if (dist > 0.0) {
+										if (dist < 25.0) {
+											reduceDecelerationCruiseAndStart = true;
+										} else {
+											dist -= 5.0;
 										}
-										edec = (spd * spd) / (2.0 * dist);
+										double edec;
+										edec = spd * spd / (2.0 * dist);
+										if (edec > dec) dec = edec;
 									} else {
-										edec = 0.0;
+										dec = BrakeDeceleration;
 									}
-									if (edec > dec) dec = edec;
 								}
 							}
 						}
@@ -1425,11 +1570,16 @@ namespace OpenBve {
 							}
 						}
 						// power / brake
-						if (dec > dectol) {
+						if (reduceDecelerationCruiseAndStart) {
+							decelerationCruise *= 0.3;
+							decelerationStart *= 0.3;
+						}
+						double brakeModeBrakeThreshold = 0.75 * decelerationStart + 0.25 * decelerationCruise;
+						if (!BrakeMode & dec > decelerationStart | BrakeMode & dec > brakeModeBrakeThreshold | forceBrakeMode) {
 							// brake
 							BrakeMode = true;
 							double decdiff = -acc - dec;
-							if (decdiff < -dectol) {
+							if (decdiff < -decelerationStep) {
 								// brake start
 								if (Train.Specs.CurrentPowerNotch.Driver == 0) {
 									TrainManager.ApplyNotch(Train, 0, true, 1, true);
@@ -1437,20 +1587,20 @@ namespace OpenBve {
 								} else {
 									TrainManager.ApplyNotch(Train, -1, true, 0, true);
 								}
-								CurrentInterval *= 0.3;
+								CurrentInterval *= 0.4;
 								if (CurrentInterval < 0.3) CurrentInterval = 0.3;
-							} else if (decdiff > dectol) {
+							} else if (decdiff > decelerationStep) {
 								// brake stop
 								TrainManager.ApplyNotch(Train, -1, true, -1, true);
 								TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Release);
-								CurrentInterval *= 0.3;
+								CurrentInterval *= 0.4;
 								if (CurrentInterval < 0.3) CurrentInterval = 0.3;
 							} else {
 								// keep brake
 								TrainManager.ApplyNotch(Train, -1, true, 0, true);
 								TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Lap);
-								CurrentInterval *= 1.3;
-								if (CurrentInterval > 1.7) CurrentInterval = 1.7;
+								CurrentInterval *= 1.2;
+								if (CurrentInterval > 1.0) CurrentInterval = 1.0;
 							}
 							if (Train.Specs.CurrentPowerNotch.Driver == 0 & Train.Specs.CurrentBrakeNotch.Driver == 0) {
 								TrainManager.ApplyHoldBrake(Train, Train.Specs.HasHoldBrake);
@@ -1458,14 +1608,16 @@ namespace OpenBve {
 							if (Train.Cars[Train.DriverCar].Specs.BrakeType == TrainManager.CarBrakeType.AutomaticAirBrake) {
 								CurrentInterval = 0.1;
 							}
-						} else if (dec > cruisedec) {
-							// cut power and brake
-							BrakeMode = true;
+						} else if (dec > decelerationCruise) {
+							// cut power/brake
+							BrakeMode = false;
 							TrainManager.ApplyNotch(Train, -1, true, -1, true);
 							TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Release);
 							if (Train.Specs.CurrentPowerNotch.Driver == 0 & Train.Specs.CurrentBrakeNotch.Driver == 0) {
 								TrainManager.ApplyHoldBrake(Train, Train.Specs.HasHoldBrake);
 							}
+							CurrentInterval *= 0.4;
+							if (CurrentInterval < 0.3) CurrentInterval = 0.3;
 						} else {
 							// power
 							BrakeMode = false;
@@ -1502,7 +1654,7 @@ namespace OpenBve {
 								TrainManager.ApplyNotch(Train, -1, true, -1, true);
 								TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Release);
 								CurrentInterval *= 0.3;
-								if (CurrentInterval < 0.3) CurrentInterval = 0.3;
+								if (CurrentInterval < 0.2) CurrentInterval = 0.2;
 							} else if (acc < acclim) {
 								// power start (under-acceleration)
 								if (Train.Specs.CurrentBrakeNotch.Driver == 0) {
@@ -1526,8 +1678,8 @@ namespace OpenBve {
 								if (Train.Specs.CurrentPowerNotch.Driver == 0 & Train.Specs.CurrentBrakeNotch.Driver == 0) {
 									TrainManager.ApplyHoldBrake(Train, Train.Specs.HasHoldBrake);
 								}
-								CurrentInterval *= 1.3;
-								if (CurrentInterval > 1.7) CurrentInterval = 1.7;
+								CurrentInterval *= 1.1;
+								if (CurrentInterval > 1.5) CurrentInterval = 1.5;
 							}
 						}
 					}
