@@ -205,7 +205,7 @@ namespace OpenBve {
 											q = false;
 											break;
 										} else if (a[j] < 0 | a[j] >= Builder.Vertices.Length) {
-											Interface.AddMessage(Interface.MessageType.Error, false, "v" + j.ToString(Culture) + " references a non-existing vertex " + Command + " at line " + (i + 1).ToString(Culture) + " in file " + FileName);
+											Interface.AddMessage(Interface.MessageType.Error, false, "v" + j.ToString(Culture) + " references a non-existing vertex in " + Command + " at line " + (i + 1).ToString(Culture) + " in file " + FileName);
 											q = false;
 											break;
 										} else if (a[j] > 65535) {
@@ -282,22 +282,6 @@ namespace OpenBve {
 								}
 								CreateCylinder(ref Builder, n, r1, r2, h);
 							} break;
-//						case "rectangle":
-//							{
-//								if (Arguments.Length > 2) {
-//									Interface.AddMessage(Interface.MessageType.Warning, false, "At most 2 arguments are expected in " + Command + " at line " + (i + 1).ToString(Culture) + " in file " + FileName);
-//								}
-//								double x = 0.0, y = 0.0;
-//								if (Arguments.Length >= 1 && Arguments[0].Length > 0 && !Interface.TryParseDoubleVb6(Arguments[0], out x)) {
-//									Interface.AddMessage(Interface.MessageType.Error, false, "Invalid argument Width in " + Command + " at line " + (i + 1).ToString(Culture) + " in file " + FileName);
-//									x = 1.0;
-//								}
-//								if (Arguments.Length >= 2 && Arguments[1].Length > 0 && !Interface.TryParseDoubleVb6(Arguments[1], out y)) {
-//									Interface.AddMessage(Interface.MessageType.Error, false, "Invalid argument Height in " + Command + " at line " + (i + 1).ToString(Culture) + " in file " + FileName);
-//									y = 1.0;
-//								}
-//								CreateRectangle(ref Builder, x, y);
-//							} break;
 						case "translate":
 						case "translateall":
 							{
@@ -809,21 +793,6 @@ namespace OpenBve {
 				}
 			}
 		}
-		
-//		// create rectangle
-//		private static void CreateRectangle(ref MeshBuilder Builder, double Width, double Height) {
-//			int v = Builder.Vertices.Length;
-//			Array.Resize<World.Vertex>(ref Builder.Vertices, v + 4);
-//			double x = 0.5 * Width;
-//			double y = 0.5 * Height;
-//			Builder.Vertices[v + 0] = new World.Vertex(new World.Vector3D(-x, -y, 0.0), new World.Vector2Df(0.0f, 1.0f));
-//			Builder.Vertices[v + 1] = new World.Vertex(new World.Vector3D(-x, y, 0.0), new World.Vector2Df(0.0f, 0.0f));
-//			Builder.Vertices[v + 2] = new World.Vertex(new World.Vector3D(x, y, 0.0), new World.Vector2Df(1.0f, 0.0f));
-//			Builder.Vertices[v + 3] = new World.Vertex(new World.Vector3D(x, -y, 0.0), new World.Vector2Df(1.0f, 1.0f));
-//			int f = Builder.Faces.Length;
-//			Array.Resize<World.MeshFace>(ref Builder.Faces, f + 1);
-//			Builder.Faces[f].Vertices = new World.MeshFaceVertex[] { new World.MeshFaceVertex(v), new World.MeshFaceVertex(v + 1), new World.MeshFaceVertex(v + 2), new World.MeshFaceVertex(v + 3) };
-//		}
 
 		// apply translation
 		private static void ApplyTranslation(MeshBuilder Builder, double x, double y, double z) {
@@ -874,7 +843,7 @@ namespace OpenBve {
 				}
 			}
 		}
-		private static void ApplyScale(ObjectManager.StaticObject Object, double x, double y, double z) {
+		internal static void ApplyScale(ObjectManager.StaticObject Object, double x, double y, double z) {
 			float rx = (float)(1.0 / x);
 			float ry = (float)(1.0 / y);
 			float rz = (float)(1.0 / z);
