@@ -72,9 +72,6 @@ namespace OpenBve {
 			internal int Reverser;
 			internal int ConstantSpeed;
 		}
-		private const int ATS_INIT_OFF = -1;
-		private const int ATS_INIT_ON_SVC = 0;
-		private const int ATS_INIT_ON_EMG = 1;
 		private const int ATS_SOUND_STOP = -10000;
 		private const int ATS_SOUND_PLAYLOOPING = 0;
 		private const int ATS_SOUND_PLAY = 1;
@@ -184,8 +181,10 @@ namespace OpenBve {
 				Spec.PowerNotches = Train.Specs.MaximumPowerNotch;
 				Spec.AtsNotch = Train.Specs.HasHoldBrake ? 2 : 1;
 				Spec.B67Notch = (int)Math.Round(0.7 * Spec.BrakeNotches);
+				if (Spec.B67Notch < 1) {
+					Spec.B67Notch = 1;
+				}
 			}
-			if (Spec.B67Notch < 1) Spec.B67Notch = 1;
 			Spec.Cars = Train.Cars.Length;
 			{
 				PluginError = true;
@@ -199,35 +198,16 @@ namespace OpenBve {
 			UpdateBrake(Train);
 			UpdateReverser(Train);
 			// finalize
-			Train.Specs.Security.Mode = TrainManager.SecuritySystem.Bve4Plugin;
+			Train.Specs.Safety.Mode = TrainManager.SafetySystem.Plugin;
 			return PluginLoadState.Successful;
 		}
 		
 		// initialize plugin
 		internal static void InitializePlugin(TrainManager.Train Train) {
 			if (!PluginLoaded) return;
-			switch (Game.TrainStart) {
-				case Game.TrainStartMode.ServiceBrakesAts:
-					PluginError = true;
-					Initialize(ATS_INIT_ON_SVC);
-					PluginError = false;
-					break;
-				case Game.TrainStartMode.EmergencyBrakesAts:
-					PluginError = true;
-					Initialize(ATS_INIT_ON_EMG);
-					PluginError = false;
-					break;
-				case Game.TrainStartMode.EmergencyBrakesNoAts:
-					PluginError = true;
-					Initialize(ATS_INIT_OFF);
-					PluginError = false;
-					break;
-				default:
-					PluginError = true;
-					Initialize(ATS_INIT_ON_SVC);
-					PluginError = false;
-					break;
-			}
+			PluginError = true;
+			Initialize((int)Game.TrainStart);
+			PluginError = false;
 		}
 		
 		// unload plugin
@@ -290,25 +270,25 @@ namespace OpenBve {
 			}
 			// process power
 			if (Handles.Power >= 0 & Handles.Power <= Train.Specs.MaximumPowerNotch) {
-				Train.Specs.CurrentPowerNotch.Security = Handles.Power;
+				Train.Specs.CurrentPowerNotch.Safety = Handles.Power;
 			} else {
-				Train.Specs.CurrentPowerNotch.Security = Train.Specs.CurrentPowerNotch.Driver;
+				Train.Specs.CurrentPowerNotch.Safety = Train.Specs.CurrentPowerNotch.Driver;
 				PluginValid = false;
 			}
-			if (Handles.Brake != 0) Train.Specs.CurrentPowerNotch.Security = 0;
+			if (Handles.Brake != 0) Train.Specs.CurrentPowerNotch.Safety = 0;
 			// process brake
-			Train.Specs.CurrentEmergencyBrake.Security = false;
+			Train.Specs.CurrentEmergencyBrake.Safety = false;
 			Train.Specs.CurrentHoldBrake.Actual = false;
 			if (Train.Cars[Train.DriverCar].Specs.BrakeType == TrainManager.CarBrakeType.AutomaticAirBrake) {
 				if (Handles.Brake == 0) {
-					Train.Specs.AirBrake.Handle.Security = TrainManager.AirBrakeHandleState.Release;
+					Train.Specs.AirBrake.Handle.Safety = TrainManager.AirBrakeHandleState.Release;
 				} else if (Handles.Brake == 1) {
-					Train.Specs.AirBrake.Handle.Security = TrainManager.AirBrakeHandleState.Lap;
+					Train.Specs.AirBrake.Handle.Safety = TrainManager.AirBrakeHandleState.Lap;
 				} else if (Handles.Brake == 2) {
-					Train.Specs.AirBrake.Handle.Security = TrainManager.AirBrakeHandleState.Service;
+					Train.Specs.AirBrake.Handle.Safety = TrainManager.AirBrakeHandleState.Service;
 				} else if (Handles.Brake == 3) {
-					Train.Specs.AirBrake.Handle.Security = TrainManager.AirBrakeHandleState.Service;
-					Train.Specs.CurrentEmergencyBrake.Security = true;
+					Train.Specs.AirBrake.Handle.Safety = TrainManager.AirBrakeHandleState.Service;
+					Train.Specs.CurrentEmergencyBrake.Safety = true;
 				} else {
 					PluginValid = false;
 				}
@@ -317,28 +297,28 @@ namespace OpenBve {
 				if (Train.Specs.HasHoldBrake) {
 					// with hold brake
 					if (Handles.Brake == Train.Specs.MaximumBrakeNotch + 2) {
-						Train.Specs.CurrentEmergencyBrake.Security = true;
-						Train.Specs.CurrentBrakeNotch.Security = Train.Specs.MaximumBrakeNotch;
+						Train.Specs.CurrentEmergencyBrake.Safety = true;
+						Train.Specs.CurrentBrakeNotch.Safety = Train.Specs.MaximumBrakeNotch;
 					} else if (Handles.Brake >= 2 & Handles.Brake <= Train.Specs.MaximumBrakeNotch + 1) {
-						Train.Specs.CurrentBrakeNotch.Security = Handles.Brake - 1;
+						Train.Specs.CurrentBrakeNotch.Safety = Handles.Brake - 1;
 					} else if (Handles.Brake == 1) {
-						Train.Specs.CurrentBrakeNotch.Security = 0;
+						Train.Specs.CurrentBrakeNotch.Safety = 0;
 						Train.Specs.CurrentHoldBrake.Actual = true;
 					} else if (Handles.Brake == 0) {
-						Train.Specs.CurrentBrakeNotch.Security = 0;
+						Train.Specs.CurrentBrakeNotch.Safety = 0;
 					} else {
-						Train.Specs.CurrentBrakeNotch.Security = Train.Specs.CurrentBrakeNotch.Driver;
+						Train.Specs.CurrentBrakeNotch.Safety = Train.Specs.CurrentBrakeNotch.Driver;
 						PluginValid = false;
 					}
 				} else {
 					// without hold brake
 					if (Handles.Brake == Train.Specs.MaximumBrakeNotch + 1) {
-						Train.Specs.CurrentEmergencyBrake.Security = true;
-						Train.Specs.CurrentBrakeNotch.Security = Train.Specs.MaximumBrakeNotch;
+						Train.Specs.CurrentEmergencyBrake.Safety = true;
+						Train.Specs.CurrentBrakeNotch.Safety = Train.Specs.MaximumBrakeNotch;
 					} else if (Handles.Brake >= 0 & Handles.Brake <= Train.Specs.MaximumBrakeNotch | Train.Specs.CurrentBrakeNotch.DelayedChanges.Length == 0) {
-						Train.Specs.CurrentBrakeNotch.Security = Handles.Brake;
+						Train.Specs.CurrentBrakeNotch.Safety = Handles.Brake;
 					} else {
-						Train.Specs.CurrentBrakeNotch.Security = Train.Specs.CurrentBrakeNotch.Driver;
+						Train.Specs.CurrentBrakeNotch.Safety = Train.Specs.CurrentBrakeNotch.Driver;
 						PluginValid = false;
 					}
 				}
@@ -576,50 +556,44 @@ namespace OpenBve {
 		// load ats config
 		internal static bool LoadAtsConfig(string TrainPath, System.Text.Encoding Encoding, TrainManager.Train Train) {
 			string atsConfigFile = Interface.GetCombinedFileName(TrainPath, "ats.cfg");
-			if (System.IO.File.Exists(atsConfigFile)) {
-				string dllTitle = System.IO.File.ReadAllText(atsConfigFile, Encoding).Trim();
-				if (dllTitle.EndsWith(".dll", StringComparison.OrdinalIgnoreCase)) {
-					string dllFile = Interface.GetCombinedFileName(TrainPath, dllTitle);
-					if (System.IO.File.Exists(dllFile)) {
-						if (Program.CurrentPlatform == Program.Platform.Windows) {
-							if (IntPtr.Size == 4) {
-								if (CheckDllHeader(dllFile)) {
-									PluginLoadState state = LoadPlugin(dllFile, Train);
-									switch (state) {
-										case PluginLoadState.CouldNotLoadDll:
-											Interface.AddMessage(Interface.MessageType.Error, false, "The train plugin " + dllTitle + " could not be loaded in " + atsConfigFile);
-											return false;
-										case PluginLoadState.InvalidPluginVersion:
-											Interface.AddMessage(Interface.MessageType.Error, false, "The train plugin " + dllTitle + " is of an unsupported version in " + atsConfigFile);
-											return false;
-										case PluginLoadState.Successful:
-											return true;
-										default:
-											return false;
-									}
-								} else {
-									Interface.AddMessage(Interface.MessageType.Error, false, "The train plugin " + dllTitle + " is of an unsupported binary format or could not be read in " + atsConfigFile);
-									return false;
-								}
-							} else {
-								Interface.AddMessage(Interface.MessageType.Information, false, "Train plugins are not supported in 64-bit environments. The built-in safety systems will be used for this train, which might not be compatible with the route.");
-								return false;
-							}
-						} else {
-							Interface.AddMessage(Interface.MessageType.Information, false, "Train plugins are not supported on operating systems other than Windows. The built-in safety systems will be used for this train, which might not be compatible with the route.");
-							return false;
-						}
-					} else {
-						Interface.AddMessage(Interface.MessageType.Error, true, "The train plugin " + dllTitle + " could not be found in " + atsConfigFile);
-						return false;
-					}
-				} else {
-					Interface.AddMessage(Interface.MessageType.Error, false, "The train plugin " + dllTitle + " must have the extension .dll in " + atsConfigFile);
-					return false;
-				}
-			} else {
+			if (!System.IO.File.Exists(atsConfigFile)) {
 				return false;
 			}
+			if (Program.CurrentPlatform != Program.Platform.Windows) {
+				Interface.AddMessage(Interface.MessageType.Information, false, "Train plugins are not supported on operating systems other than Windows. The built-in safety systems will be used for this train, which might not be compatible with the route.");
+				return false;
+			}
+			if (IntPtr.Size != 4) {
+				Interface.AddMessage(Interface.MessageType.Information, false, "Train plugins are not supported in 64-bit environments. The built-in safety systems will be used for this train, which might not be compatible with the route.");
+				return false;
+			}
+			string dllTitle = System.IO.File.ReadAllText(atsConfigFile, Encoding).Trim();
+			if (!dllTitle.EndsWith(".dll", StringComparison.OrdinalIgnoreCase)) {
+				Interface.AddMessage(Interface.MessageType.Error, false, "The train plugin " + dllTitle + " must have the extension .dll in " + atsConfigFile);
+				return false;
+			}
+			string dllFile = Interface.GetCombinedFileName(TrainPath, dllTitle);
+			if (!System.IO.File.Exists(dllFile)) {
+				Interface.AddMessage(Interface.MessageType.Error, true, "The train plugin " + dllTitle + " could not be found in " + atsConfigFile);
+				return false;
+			}
+			if (!CheckDllHeader(dllFile)) {
+				Interface.AddMessage(Interface.MessageType.Error, false, "The train plugin " + dllTitle + " is of an unsupported binary format or could not be read in " + atsConfigFile);
+				return false;
+			}
+			PluginLoadState state = LoadPlugin(dllFile, Train);
+			switch (state) {
+				case PluginLoadState.CouldNotLoadDll:
+					Interface.AddMessage(Interface.MessageType.Error, false, "The train plugin " + dllTitle + " could not be loaded in " + atsConfigFile);
+					return false;
+				case PluginLoadState.InvalidPluginVersion:
+					Interface.AddMessage(Interface.MessageType.Error, false, "The train plugin " + dllTitle + " is of an unsupported version in " + atsConfigFile);
+					return false;
+				case PluginLoadState.Successful:
+					return true;
+				default:
+					return false;
+			}
 		}
 		
 		// check dll header
